# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Rick Spencer rick.spencer@canonical.com
#This program is free software: you can redistribute it and/or modify it 
#under the terms of the GNU General Public License version 3, as published 
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but 
#WITHOUT ANY WARRANTY; without even the implied warranties of 
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along 
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import sys
import os
import gtk
from desktopcouch.records.server import CouchDatabase
from desktopcouch.records.record import Record

from lolz.lolzconfig import getdatapath

class PreferencesLolzDialog(gtk.Dialog):
    __gtype_name__ = "PreferencesLolzDialog"
    prefernces = {}

    def __init__(self):
        """__init__ - This function is typically not called directly.
        Creation of a PreferencesLolzDialog requires redeading the associated ui
        file and parsing the ui definition extrenally,
        and then calling PreferencesLolzDialog.finish_initializing().

        Use the convenience function NewPreferencesLolzDialog to create
        NewAboutLolzDialog objects.
        """

        pass

    def finish_initializing(self, builder):
        """finish_initalizing should be called after parsing the ui definition
        and creating a AboutLolzDialog object with it in order to finish
        initializing the start of the new AboutLolzDialog instance.
        """

        #get a reference to the builder and set up the signals
        self.builder = builder
        self.builder.connect_signals(self)

        #set up couchdb and the preference info
        self.__db_name = "lolz"
        self.__database = CouchDatabase(self.__db_name, create=True)
        self.__preferences = None
        self.__key = None

        #set the record type and then initalize the preferences
        self.__record_type = "http://wiki.ubuntu.com/Quickly/RecordTypes/Lolz/Preferences"
        self.__preferences = self.get_preferences()
        do_cats = self.__preferences["cats"]
        self.builder.get_object("checkbutton_cats").set_active(do_cats)
        do_dogs = self.__preferences["dogs"]
        self.builder.get_object("checkbutton_dogs").set_active(do_dogs)
        #TODO:code for other initialization actions should be added here

    def get_preferences(self):
        """get_preferences  -returns a dictionary object that contain
        preferences for lolz. Creates a couchdb record if
        necessary.
        """

        if self.__preferences == None: #the dialog is initializing
            self.__load_preferences()
            
        #if there were no saved preference, this 
        return self.__preferences

    def __load_preferences(self):
        #TODO: add prefernces to the self.__preferences dict
        #default preferences that will be overwritten if some are saved
        default_preferences = {"record_type":self.__record_type,
                               "cats":True,
                               "dogs":True,
                               "fails":True}

        self.__preferences = default_preferences
        
        results = self.__database.get_records(record_type=self.__record_type, create_view=True)
       
        if len(results.rows) == 0:
            #no preferences have ever been saved
            #save them before returning
            self.__key = self.__database.put_record(Record(self.__preferences))
        else:
            self.__preferences = results.rows[0].value
            self.__key = results.rows[0].value["_id"]
            
            for pref in default_preferences.keys():
                if pref not in self.__preferences:
                    #create the pref
                    self.__database.update_fields(self.__key,{pref:True})
            
    def __save_preferences(self):
        self.__database.update_fields(self.__key, self.__preferences)

    def ok(self, widget, data=None):
        """ok - The user has elected to save the changes.
        Called before the dialog returns gtk.RESONSE_OK from run().
        """

        do_cats = self.builder.get_object("checkbutton_cats").get_active()
        self.__preferences["cats"] = do_cats
        do_dogs = self.builder.get_object("checkbutton_dogs").get_active()
        self.__preferences["dogs"] = do_dogs

        #make any updates to self.__preferences here
        #self.__preferences["preference1"] = "value2"
        self.__save_preferences()

    def cancel(self, widget, data=None):
        """cancel - The user has elected cancel changes.
        Called before the dialog returns gtk.RESPONSE_CANCEL for run()
        """

        #restore any changes to self.__preferences here
        pass

def NewPreferencesLolzDialog():
    """NewPreferencesLolzDialog - returns a fully instantiated
    PreferencesLolzDialog object. Use this function rather than
    creating a PreferencesLolzDialog instance directly.
    """

    #look for the ui file that describes the ui
    ui_filename = os.path.join(getdatapath(), 'ui', 'PreferencesLolzDialog.ui')
    if not os.path.exists(ui_filename):
        ui_filename = None

    builder = gtk.Builder()
    builder.add_from_file(ui_filename)
    dialog = builder.get_object("preferences_lolz_dialog")
    dialog.finish_initializing(builder)
    return dialog

if __name__ == "__main__":
    dialog = NewPreferencesLolzDialog()
    dialog.show()
    gtk.main()

